@@ -131,6 +131,22 @@ struct PostItemCreator {
 	}
 };
 
+
+struct StringListSetter: public ArgHandler {
+	QList<QString> & dst;
+	StringListSetter(QList<QString> & a, QString valueName) : dst(a) {
+		argn.push_back(valueName);
+	}
+	virtual bool operator() (const char ** args, CommandLineParserPrivate &) {
+		dst.append( args[0] );
+		return true;
+	}
+	virtual void useDefault() {
+		dst.clear();
+	}
+};
+
+
 /*!
   Putting values into a map
 */
@@ -245,7 +261,7 @@ typedef SomeSetter<StrTM> StrSetter;
 struct QStrTM: public SomeSetterTM<QString> {
 	static QString strToT(const char * val, bool & ok) {
 		ok=true;
-		return QString(val);
+		return QString::fromLocal8Bit(val);
 	}
 	static QString TToStr(QString t, bool & ok) {
 		ok=!t.isEmpty();
@@ -452,7 +468,6 @@ CommandLineParserPrivate::CommandLineParserPrivate(Settings & s):
 	addarg("proxy",'p',"Use a proxy", new ProxySetter(s.proxy, "proxy"));
 	addarg("username",0,"HTTP Authentication username", new QStrSetter(s.username, "username",""));
 	addarg("password",0,"HTTP Authentication password", new QStrSetter(s.password, "password",""));
-	addarg("custom-header",0,"Set an additional HTTP header (repeatable)", new MapSetter<>(s.customHeaders, "name", "value"));
 	qthack(true);
 	addarg("book",'b',"Set the options one would usually set when printing a book", new Caller<BookFunc>());
 	addarg("cover",0,"Use html document as cover. It will be inserted before the toc with no headers and footers",new QStrSetter(s.cover,"url",""));
@@ -461,6 +476,8 @@ CommandLineParserPrivate::CommandLineParserPrivate(Settings & s):
 	qthack(false);
 	
 	extended(true);
+	addarg("ignore-load-errors", 0, "Ignore pages that claimes to have encountered an error during loading", new ConstSetter<bool>(s.ignoreLoadErrors, true, false));
+	addarg("custom-header",0,"Set an additional HTTP header (repeatable)", new MapSetter<>(s.customHeaders, "name", "value"));
 	addarg("manpage", 0, "Output program man page", new Caller<ManPageFunc>());
 	addarg("htmldoc", 0, "Output program html help", new Caller<ReadmeFunc<true> >());
 	addarg("readme", 0, "Output program readme", new Caller<ReadmeFunc<false> >());
@@ -477,19 +494,25 @@ CommandLineParserPrivate::CommandLineParserPrivate(Settings & s):
 	addarg("redirect-delay",0,"Wait some milliseconds for js-redirects", new IntSetter(s.jsredirectwait,"msec",200));
 	addarg("enable-plugins",0,"Enable installed plugins (such as flash", new ConstSetter<bool>(s.enablePlugins,true,false));
 	addarg("zoom",0,"Use this zoom factor", new FloatSetter(s.zoomFactor,"float",1.0));
+	addarg("minimum-font-size",0,"Minimum font size", new IntSetter(s.minimumFontSize,"int",5));
 	addarg("read-args-from-stdin",0,"Read command line arguments from stdin", new ConstSetter<bool>(s.readArgsFromStdin,true,false));
 	addarg("cookie-jar", 0, "Read and write cookies from and to the supplied cookie jar file", new QStrSetter(s.cookieJar, "path", "") );
 	addarg("cookie",0,"Set an additional cookie (repeatable)", new MapSetter<>(s.cookies, "name", "value"));
-
 	addarg("post", 0, "Add an additional post field (repeatable)", new MapSetter<PostItemCreator<false> >(s.post, "name", "value"));
 	addarg("post-file", 0, "Post an aditional file (repeatable)", new MapSetter<PostItemCreator<true> >(s.post, "name", "path"));
-
+	addarg("title", 0, "The title of the generated pdf file (The title of the first document is used if not specified)", new QStrSetter(s.documentTitle,"text",""));
+	addarg("disallow-local-file-access", 0, "Do not allowed conversion of a local file to read in other local files, unless explecitily allowed with --allow", new ConstSetter<bool>(s.blockLocalFileAccess, true, false));
+	addarg("allow", 0, "Allow the file or files from the specified folder to be loaded (repeatable)", new StringListSetter(s.allowed,"path"));
+	
 	qthack(true);
+	addarg("forms", 0, "Turn HTML form fields into pdf form fields", new ConstSetter<bool>(s.produceForms, true, false));
 	addarg("disable-internal-links",0,"Do no make local links", new ConstSetter<bool>(s.useLocalLinks,false,true));
 	addarg("disable-external-links",0,"Do no make links to remote web pages", new ConstSetter<bool>(s.useExternalLinks,false,true));
 	addarg("print-media-type",0,"Use print media-type instead of screen", new ConstSetter<bool>(s.printMediaType,true,false));
 	addarg("page-offset",0,"Set the starting page number", new IntSetter(s.pageOffset,"offset",1));
 	addarg("disable-smart-shrinking", 0, "Disable the intelligent shrinking strategy used by WebKit that makes the pixel/dpi ratio none constant",new ConstSetter<bool>(s.enableIntelligentShrinking, false, true));
+	addarg("replace",0, "Replace [name] with value in header and footer (repeatable)", new MapSetter<>(s.replacements, "name", "value"));
+	addarg("disable-pdf-compression", 0 , "Do not use lossless compression on pdf objects", new ConstSetter<bool>(s.useCompression,false,true));
 #ifdef Q_WS_X11
 	addarg("use-xserver",0,"Use the X server (some plugins and other stuff might not work without X11)", new ConstSetter<bool>(s.useGraphics,true,false));
 #endif
@@ -504,7 +527,10 @@ CommandLineParserPrivate::CommandLineParserPrivate(Settings & s):
 	addarg("no-background",0,"Do not print background", new ConstSetter<bool>(s.background,false,true));
 	addarg("user-style-sheet",0,"Specify a user style sheet, to load with every page", new QStrSetter(s.userStyleSheet,"url",""));
 #endif
-	
+	addarg("debug-javascript", 0,"Show javascript debugging output", new ConstSetter<bool>(s.debugJavascript, true, false));
+#if QT_VERSION >= 0x040600
+	addarg("stop-slow-scripts", 0, "Stop slow running javascripts", new ConstSetter<bool>(s.stopSlowScripts, true, false));
+#endif	
 	extended(false);
 	section("Headers And Footer Options");
 	qthack(true);
@@ -516,7 +542,7 @@ CommandLineParserPrivate::CommandLineParserPrivate(Settings & s):
 	addarg("footer-line",0,"Display line above the footer", new ConstSetter<bool>(s.footer.line,true,false));
 	addarg("footer-right",0,"Right aligned footer text", new QStrSetter(s.footer.right,"text",""));
 	addarg("footer-spacing",0,"Spacing between footer and content in mm", new FloatSetter(s.footer.spacing,"real",0.0));
-	addarg("footer-html",0,"Adds a html header", new QStrSetter(s.footer.htmlUrl,"url",""));
+	addarg("footer-html",0,"Adds a html footer", new QStrSetter(s.footer.htmlUrl,"url",""));
 	addarg("header-center",0,"Centered header text", new QStrSetter(s.header.center,"text",""));
 	addarg("header-font-name",0,"Set header font name", new QStrSetter(s.header.fontName,"name","Arial"));
 	addarg("header-font-size",0,"Set header font size", new IntSetter(s.header.fontSize,"size",11));
@@ -535,7 +561,8 @@ CommandLineParserPrivate::CommandLineParserPrivate(Settings & s):
 	addarg("toc-no-dots",0,"Do not use dots, in the toc", new ConstSetter<bool>(s.toc.useDots,false,true));
 	addarg("toc-depth",0,"Set the depth of the toc", new IntSetter(s.toc.depth,"level",3));
 	addarg("toc-header-text",0,"The header text of the toc", new QStrSetter(s.toc.captionText,"text","Table Of Contents"));
-	addarg("toc-header-fs",0,"The font size of the toc header", new IntSetter(s.toc.captionFontSize,"size",15));
+	addarg("toc-header-font-size",0,"The font size of the toc header", new IntSetter(s.toc.captionFontSize,"size",15));
+	addarg("toc-header-font-name",0,"The font of the toc header (if unset use --toc-font-name)", new QStrSetter(s.toc.captionFontName,"name"));
 	addarg("toc-disable-links",0,"Do not link from toc to sections", new ConstSetter<bool>(s.toc.forwardLinks,false, true));
 	addarg("toc-disable-back-links",0,"Do not link from section header to toc", new ConstSetter<bool>(s.toc.backLinks,false,true));
 	for (uint i=0; i < Settings::TOCSettings::levels; ++i) {
@@ -550,6 +577,7 @@ CommandLineParserPrivate::CommandLineParserPrivate(Settings & s):
 	qthack(true);
 	addarg("outline",0,"Put an outline into the pdf", new ConstSetter<bool>(s.outline,true,false));
 	addarg("outline-depth",0,"Set the depth of the outline", new IntSetter(s.outlineDepth,"level",4));
+	addarg("dump-outline",0,"Dump the outline to a file",new QStrSetter(s.dumpOutline,"file",""));
 	qthack(true);
 	extended(false);
 }
