@@ -25,16 +25,18 @@
 */
 void CommandLineParserPrivate::outputName(Outputter * o) const {
 	o->beginSection("Name");
-	o->beginParagraph();
-	o->text(QString("wkhtmltopdf ")+QString::number(MAJOR_VERSION)+"."+QString::number(MINOR_VERSION)+"."+QString::number(PATCH_VERSION)+(QString(STRINGIZE(BUILD)).isEmpty()?"":" ")+STRINGIZE(BUILD));
-	o->text(", ");
-#ifdef __EXTENSIVE_WKHTMLTOPDF_QT_HACK__
-	o->text("using wkhtmltopdf patched qt.");
-#else
-	o->bold("not");
-	o->text(" using wkhtmltopdf patched qt.");
-#endif
-	o->endParagraph();
+	o->paragraph(QString("wkhtmltopdf ")+QString::number(MAJOR_VERSION)+"."+QString::number(MINOR_VERSION)+"."+QString::number(PATCH_VERSION)+(QString(STRINGIZE(BUILD)).isEmpty()?"":" ")+STRINGIZE(BUILD));
+	o->endSection();
+}
+
+
+/*!
+  Output name and a short description
+  \param o The outputter to output to
+*/
+void CommandLineParserPrivate::outputManName(Outputter * o) const {
+	o->beginSection("Name");
+	o->paragraph("wkhtmltopdf - html to pdf converter");
 	o->endSection();
 }
 
@@ -83,8 +85,15 @@ void CommandLineParserPrivate::outputSynopsis(Outputter * o) const {
 */
 void CommandLineParserPrivate::outputDescripton(Outputter * o) const {
 	o->beginSection("Description");
-	o->paragraph(
-		"Converts one or more HTML pages into a PDF document.");
+	o->beginParagraph();
+	o->text("Converts one or more HTML pages into a PDF document, ");
+#ifdef __EXTENSIVE_WKHTMLTOPDF_QT_HACK__
+	o->text("using wkhtmltopdf patched qt.");
+#else
+	o->bold("not");
+	o->text(" using wkhtmltopdf patched qt.");
+#endif
+	o->endParagraph();
 	o->endSection();
 }
 
@@ -127,7 +136,7 @@ void CommandLineParserPrivate::outputPageBreakDoc(Outputter * o) const {
 	o->beginSection("Page Breaking");
 	o->paragraph(
 		"The current page breaking algorithm of WebKit leaves much to be desired. "
-		"Basically webkit will render everything into one long page, and the cut it up "
+		"Basically webkit will render everything into one long page, and then cut it up "
 		"into pages. This means that if you have two columns of text where one is "
 		"vertically shifted by half a line. Then webkit will cut a line into to pieces "
 		"display the top half on one page. And the bottom half on another page. "
@@ -185,6 +194,8 @@ void CommandLineParserPrivate::outputHeaderFooterDoc(Outputter * o) const {
 " * [webpage]    Replaced by the URL of the page being printed\n"
 " * [section]    Replaced by the name of the current section\n"
 " * [subsection] Replaced by the name of the current subsection\n"
+" * [date]       Replaced by the current date in system local format\n"
+" * [time]       Replaced by the current time in system local format\n"
 "\n");
 	o->paragraph("As an example specifying --header-right \"Page [page] of [toPage]\", "
 				 "will result in the text \"Page x of y\" where x is the number of the "
@@ -216,7 +227,7 @@ void CommandLineParserPrivate::outputHeaderFooterDoc(Outputter * o) const {
 "</body></html>\n"
 "\n"
 		);
-	o->paragraph("As can be seen from the example, the arguments are send to the header/footer html "
+	o->paragraph("As can be seen from the example, the arguments are sent to the header/footer html "
 				 "documents in get fashion.");
 	o->endSection();
 }
@@ -340,7 +351,7 @@ void CommandLineParserPrivate::outputInstallation(Outputter * o) const {
 	o->beginSection("Installation");
 	o->paragraph(
 		"There are several ways to install wkhtmltopdf.  You can download a "
-		"already compiled binary, or you can compile wkhtmltopdf your self. "
+		"already compiled binary, or you can compile wkhtmltopdf yourself. "
 		"On windows the easiest way to install wkhtmltopdf is to download "
 		"the latest installer. On linux you can download the latest static "
 		"binary, however you still need to install some other pieces of "
@@ -402,7 +413,7 @@ void CommandLineParserPrivate::outputStaticProblems(Outputter * o) const {
 	o->text("Unfortunately the static binary is not particularly static, on Linux it depends "
 			"on both glibc and openssl, furthermore you will need to have an xserver installed "
 			"but not necessary running. You will need to have different fonts install including "
-			"xfonts-scalable (Type1).  See ");
+			"xfonts-scalable (Type1), and msttcorefonts.  See ");
 	o->link("http://code.google.com/p/wkhtmltopdf/wiki/static");
 	o->text(" for trouble shouting.");
 	o->endParagraph();
