@@ -1,4 +1,4 @@
-//-*- mode: c++; tab-width: 4; indent-tabs-mode: t; c-file-style: "stroustrup"; -*-
+
 // This file is part of wkhtmltopdf.
 //
 // wkhtmltopdf is free software: you can redistribute it and/or modify
@@ -19,11 +19,13 @@
 #include <QFile>
 #include <QPair>
 #include <QTimer>
+#include <QPrintEngine>
 #include <QWebFrame>
 #include <QWebPage>
 #include <QWebSettings>
 #include <qapplication.h>
 #include <qfileinfo.h>
+#include <QDateTime>
 #ifdef Q_OS_WIN32
 #include <io.h>
 #include <fcntl.h>
@@ -39,6 +41,11 @@
   \brief Defines the PageConverterPrivate class
 */
 
+bool looksLikeHtmlAndNotAUrl(QString str) {
+	QString s = str.split("?")[0];
+	return s.count('<') > 0 || s.count('<') > 0;
+}
+
 PageConverterPrivate::PageConverterPrivate(Settings & s, PageConverter & o) :
 	settings(s), pageLoader(s),
 	outer(o), printer(0), painter(0)
@@ -70,6 +77,8 @@ PageConverterPrivate::PageConverterPrivate(Settings & s, PageConverter & o) :
 	QWebSettings::globalSettings()->setAttribute(QWebSettings::JavascriptCanOpenWindows, false);
 	QWebSettings::globalSettings()->setAttribute(QWebSettings::JavascriptCanAccessClipboard, false);
 
+	QWebSettings::globalSettings()->setFontSize(QWebSettings::MinimumFontSize, s.minimumFontSize);
+
 #if QT_VERSION >= 0x040500
 	//Newer vertions of QT have even more settings to change
 	QWebSettings::globalSettings()->setAttribute(QWebSettings::PrintElementBackgrounds, settings.background);
@@ -122,6 +131,18 @@ void PageConverterPrivate::beginConvert() {
 
   	if (!settings.cover.isEmpty())
 		settings.in.push_front(settings.cover);
+
+	if (!settings.header.htmlUrl.isEmpty() && looksLikeHtmlAndNotAUrl(settings.header.htmlUrl)) {
+		emit outer.error("--header-html should be a URL and not a string containing HTML code.");
+		fail();
+		return;
+	}
+
+	if (!settings.footer.htmlUrl.isEmpty() && looksLikeHtmlAndNotAUrl(settings.footer.htmlUrl)) {
+		emit outer.error("--header-html should be a URL and not a string containing HTML code.");
+		fail();
+		return;
+	}
 	
 #ifndef __EXTENSIVE_WKHTMLTOPDF_QT_HACK__	
 	if (settings.in.size() > 1) {
@@ -145,6 +166,8 @@ void PageConverterPrivate::fail() {
 	convertionDone = true;
 	clearResources();
 	emit outer.finished(false);
+	
+	qApp->exit(0); // quit qt's event handling
 }
 
 /*!
@@ -199,6 +222,7 @@ void PageConverterPrivate::preparePrint(bool ok) {
 	printer->setOrientation(settings.orientation);
 	printer->setColorMode(settings.colorMode);
 
+
 	if (!printer->isValid()) {
 		emit outer.error("Unable to write to destination");
 		fail();
@@ -207,11 +231,19 @@ void PageConverterPrivate::preparePrint(bool ok) {
 
 #ifndef __EXTENSIVE_WKHTMLTOPDF_QT_HACK__
 	//If you do not have the hacks you get this crappy solution
+
 	printer->setCollateCopies(settings.copies);
 	printer->setCollateCopies(settings.collate);
 	printPage(true);
 #else
+	printer->printEngine()->setProperty(QPrintEngine::PKK_UseCompression, settings.useCompression);
+
 	painter = new QPainter();
+	
+	QString title = settings.documentTitle;
+	if (title == "") 
+		title = pages[0]->mainFrame()->title();
+	printer->setDocName(title);
 	if (!painter->begin(printer)) {
 		emit outer.error("Unable to write to destination");
 		fail();
@@ -232,35 +264,9 @@ void PageConverterPrivate::preparePrint(bool ok) {
 			urlToDoc[ pages[d]->mainFrame()->url().toString(QUrl::RemoveFragment) ]  = d;
 		
 		for(int d=0; d < pages.size(); ++d) {
-
 			progressString = QString("Page ")+QString::number(d+1)+QString(" of ")+QString::number(pages.size());
 			emit outer.progressChanged((d+1)*100 / pages.size());
-			
-			foreach(const QWebElement & elm,pages[d]->mainFrame()->findAllElements("a")) {
-				QUrl href=QUrl(elm.attribute("href"));
-				if(href.isEmpty()) continue;
-				href=pages[d]->mainFrame()->url().resolved(href);
-				if(urlToDoc.contains(href.toString(QUrl::RemoveFragment))) {
-					if(settings.useLocalLinks) {
-						int t = urlToDoc[href.toString(QUrl::RemoveFragment)];
-						QWebElement e;
-						if(!href.hasFragment()) 
-							e = pages[t]->mainFrame()->findFirstElement("body");
-						else {
-							e = pages[t]->mainFrame()->findFirstElement("a[name=\""+href.fragment()+"\"]");
-							if(e.isNull()) 
-								e = pages[t]->mainFrame()->findFirstElement("*[id=\""+href.fragment()+"\"]");
-						}
-						if(e.isNull())
-							qDebug() << "Unable to find target for local link " << href; 
-						else {
-							anchors[t][href.toString()] = e;
-							localLinks[d].push_back( qMakePair(elm, href.toString()) );
-						}
-					}
-				} else if(settings.useExternalLinks)
-					externalLinks[d].push_back( qMakePair(elm, href.toString() ) );
-			}
+			findLinks(pages[d]->mainFrame(), localLinks[d], externalLinks[d]);
 		}
 	}
 
@@ -307,12 +313,13 @@ void PageConverterPrivate::preparePrint(bool ok) {
 	headers.clear();
 	footers.clear();
 	if(!settings.header.htmlUrl.isEmpty() || !settings.footer.htmlUrl.isEmpty()) {
+		QWebSettings::globalSettings()->setAttribute(QWebSettings::JavascriptEnabled, true);
 		currentPhase = 3;
 		emit outer.phaseChanged();
 		for(int p=0; p < outline->pageCount(); ++p) {
 			QHash<QString, QString> parms;
-			outline->fillHeaderFooterParms(page, parms);
-
+			fillParms(parms, page);
+			
 			if(!settings.header.htmlUrl.isEmpty())
 				headers.push_back(loadHeaderFooter(settings.header.htmlUrl, parms) );
 			if(!settings.footer.htmlUrl.isEmpty())
@@ -326,6 +333,42 @@ void PageConverterPrivate::preparePrint(bool ok) {
 }
 
 #ifdef __EXTENSIVE_WKHTMLTOPDF_QT_HACK__
+void PageConverterPrivate::findLinks(QWebFrame * frame, QVector<QPair<QWebElement, QString> > & local, QVector<QPair<QWebElement, QString> > & external) {
+	if (!settings.useLocalLinks && !settings.useExternalLinks) return;
+	foreach (const QWebElement & elm, frame->findAllElements("a")) {
+		QUrl href=QUrl(elm.attribute("href"));
+		if (href.isEmpty()) continue;
+		href=frame->url().resolved(href);
+		if (urlToDoc.contains(href.toString(QUrl::RemoveFragment))) {
+			if (settings.useLocalLinks) {
+				int t = urlToDoc[href.toString(QUrl::RemoveFragment)];
+				QWebElement e;
+				if (!href.hasFragment()) 
+					e = pages[t]->mainFrame()->findFirstElement("body");
+				else {
+					e = pages[t]->mainFrame()->findFirstElement("a[name=\""+href.fragment()+"\"]");
+					if(e.isNull()) 
+						e = pages[t]->mainFrame()->findFirstElement("*[id=\""+href.fragment()+"\"]");
+				}
+				if (e.isNull())
+					qDebug() << "Unable to find target for local link " << href; 
+				else {
+					anchors[t][href.toString()] = e;
+					local.push_back( qMakePair(elm, href.toString()) );
+				}
+			}
+		} else if (settings.useExternalLinks)
+			external.push_back( qMakePair(elm, href.toString() ) );
+	}
+}
+
+void PageConverterPrivate::fillParms(QHash<QString, QString> & parms, int page) {
+	outline->fillHeaderFooterParms(page, parms);
+	QDateTime t(QDateTime::currentDateTime());
+	parms["time"] = t.time().toString(Qt::SystemLocaleShortDate);
+	parms["date"] = t.date().toString(Qt::SystemLocaleShortDate);
+}
+
 void PageConverterPrivate::beginPage(int & actualPage, bool & first) {
 	progressString = QString("Page ") + QString::number(actualPage) + QString(" of ") + QString::number(actualPages);
 	emit outer.progressChanged(actualPage * 100 / actualPages);
@@ -337,10 +380,11 @@ void PageConverterPrivate::beginPage(int & actualPage, bool & first) {
 }
 
 void PageConverterPrivate::endPage(bool actual, bool hasHeaderFooter) {
+	typedef QPair<QWebElement, QString> p_t;		
+		
 	if(hasHeaderFooter && actual) {
-
 		QHash<QString, QString> parms;
-		outline->fillHeaderFooterParms(logicalPage, parms);
+		fillParms(parms, logicalPage);
 
 		//Webkit used all kinds of crazy coordinate transformation, and font setup
 		//We save it here and restore some sane defaults
@@ -364,7 +408,7 @@ void PageConverterPrivate::endPage(bool actual, bool hasHeaderFooter) {
 		
 		spacing = settings.footer.spacing * printer->height() / printer->heightMM();
 		//IF needed draw the footer line
-		if (settings.footer.line) painter->drawLine(0, h + spacing, w, h);
+		if (settings.footer.line) painter->drawLine(0, h + spacing, w, h + spacing);
 		//Guess the height of the footer text
 		painter->setFont(QFont(settings.footer.fontName, settings.footer.fontSize));
 		dy = painter->boundingRect(0, 0, w, h, Qt::AlignTop, "M").height();
@@ -385,9 +429,22 @@ void PageConverterPrivate::endPage(bool actual, bool hasHeaderFooter) {
 		painter->translate(0, -spacing);
 		QWebPrinter wp(headers[logicalPage-1]->mainFrame(), printer, *painter);
 		painter->translate(0,-wp.elementLocation(headers[logicalPage-1]->mainFrame()->findFirstElement("body")).second.height());
+
+		QVector<p_t> local;
+		QVector<p_t> external;
+		findLinks(headers[logicalPage-1]->mainFrame(), local, external);
+		foreach(const p_t & p, local) {
+			QRectF r = wp.elementLocation(p.first).second;
+			painter->addLink(r, p.second);
+		}
+		foreach(const p_t & p, external) {
+			QRectF r = wp.elementLocation(p.first).second;
+			qDebug() << p.second;
+			painter->addHyperlink(r, QUrl(p.second));
+		}
 		wp.spoolPage(1);
 		painter->restore();
-	}
+ 	}
 
 	if(actual && logicalPage <= footers.size()) {
 		painter->save();
@@ -395,6 +452,19 @@ void PageConverterPrivate::endPage(bool actual, bool hasHeaderFooter) {
 		double spacing = settings.footer.spacing * printer->height() / printer->heightMM();
 		painter->translate(0, printer->height()+ spacing);
 		QWebPrinter wp(footers[logicalPage-1]->mainFrame(), printer, *painter);
+		
+		QVector<p_t> local;
+		QVector<p_t> external;
+		findLinks(footers[logicalPage-1]->mainFrame(), local, external);
+		foreach(const p_t & p, local) {
+			QRectF r = wp.elementLocation(p.first).second;
+			painter->addLink(r, p.second);
+		}
+		foreach(const p_t & p, external) {
+			QRectF r = wp.elementLocation(p.first).second;
+			qDebug() << p.second;
+			painter->addHyperlink(r, QUrl(p.second));
+		}
 		wp.spoolPage(1);
 		painter->restore();
 	}
@@ -432,7 +502,7 @@ void PageConverterPrivate::printPage(bool ok) {
 
 	progressString = "Preparing";
 	emit outer.progressChanged(0);
-	
+			
 	for(int cc_=0; cc_ < cc; ++cc_) {
 		logicalPage=1;
 		for(int d=0; d < pages.size(); ++d) {
@@ -479,11 +549,42 @@ void PageConverterPrivate::printPage(bool ok) {
 			for(QVector< QPair<QWebElement,QString> >::iterator i=externalLinks[d].begin();
 				i != externalLinks[d].end(); ++i)
 				myExternalLinks[wp.elementLocation(i->first).first].push_back(*i);
-				
+
+			QHash<int, QVector<QWebElement> > myFormElements;
+			if (settings.produceForms) {
+				foreach(const QWebElement & elm,pages[d]->mainFrame()->findAllElements("input")) 
+					myFormElements[wp.elementLocation(elm).first].push_back(elm);
+				foreach(const QWebElement & elm,pages[d]->mainFrame()->findAllElements("textarea")) 
+					myFormElements[wp.elementLocation(elm).first].push_back(elm);
+			}
 			for(int p=0; p < wp.pageCount(); ++p) {
 				for(int pc_=0; pc_ < pc; ++pc_) {
 					beginPage(actualPage,first);
 					wp.spoolPage(p+1);
+
+					foreach(const QWebElement & elm, myFormElements[p+1]) {
+						QString type = elm.attribute("type");
+						QString tn = elm.tagName();
+						QString name = elm.attribute("name");
+						if (tn == "TEXTAREA" || type == "text" || type == "password") {
+							painter->addTextField(
+								wp.elementLocation(elm).second,
+								tn == "TEXTAREA"?elm.toPlainText():elm.attribute("value"),
+								name, 
+								tn == "TEXTAREA",		   
+								type == "password",
+								elm.attribute("readonly") == "readonly", 
+								elm.hasAttribute("maxlength")?elm.attribute("maxlength").toInt():-1
+								);
+						} else if (type == "checkbox") {
+							painter->addCheckBox(
+								wp.elementLocation(elm).second,
+								elm.attribute("checked") == "checked",
+								name,
+								elm.attribute("readonly") == "readonly");
+						}
+					}
+
 					for(QHash<QString, QWebElement>::iterator i=myAnchors[p+1].begin();
 						i != myAnchors[p+1].end(); ++i) {
 						QRectF r = wp.elementLocation(i.value()).second;
@@ -507,6 +608,7 @@ void PageConverterPrivate::printPage(bool ok) {
 		}
  	}
 	outline->printOutline(printer);
+	outline->dumpOutline();
  	painter->end();
 #endif
 	if (settings.out == "-" && lout != "/dev/stdout") {
@@ -532,6 +634,8 @@ void PageConverterPrivate::printPage(bool ok) {
 	emit outer.phaseChanged();
 	convertionDone = true;
 	emit outer.finished(true);
+	
+  qApp->exit(0); // quit qt's event handling
 }
 
 #ifdef __EXTENSIVE_WKHTMLTOPDF_QT_HACK__
@@ -559,8 +663,10 @@ QString PageConverterPrivate::hfreplace(const QString & q, const QHash<QString,
 bool PageConverterPrivate::convert() {
 	convertionDone=false;
 	beginConvert();
-	while(!convertionDone)
-		qApp->processEvents(QEventLoop::WaitForMoreEvents | QEventLoop::AllEvents);
+	while(!convertionDone) {
+    qApp->exec();
+	  //qApp->processEvents(QEventLoop::WaitForMoreEvents | QEventLoop::AllEvents);
+	}
 	return !error;
 }
 
